@@ -23,7 +23,7 @@ use vars qw'$VERSION';
 use Carp;
 use Text::BibTeX qw(:metatypes :nodetypes);
 
-$VERSION = 0.74;
+$VERSION = 0.78;
 
 =head1 NAME
 
@@ -31,7 +31,7 @@ Text::BibTeX::Entry - read and parse BibTeX files
 
 =head1 SYNOPSIS
 
-   use Text::BibTeX;            # do not use Text::BibTeX::Entry alone!
+   use Text::BibTeX::Entry;
 
    # ...assuming that $bibfile and $newbib are both objects of class
    # Text::BibTeX::File, opened for reading and writing (respectively):
@@ -127,7 +127,7 @@ anything extra.
 
 =over 4
 
-=item new ([SOURCE])
+=item new ([OPTS ,] [SOURCE])
 
 Creates a new C<Text::BibTeX::Entry> object.  If the SOURCE parameter is
 supplied, it must be one of the following: a C<Text::BibTeX::File> (or
@@ -169,6 +169,28 @@ But using a C<Text::BibTeX::File> object is simpler and preferred:
 Returns the new object, unless SOURCE is supplied and reading/parsing
 the entry fails (e.g., due to end of file) -- then it returns false.
 
+You may supply a reference to an option hash as first argument.
+Supported options are:
+
+=over 4 
+
+=item BINMODE
+
+Set the way Text::BibTeX deals with strings. By default it manages
+strings as bytes. You can set BINMODE to 'utf-8' to get NFC normalized
+
+Text::BibTeX::Entry->new(
+      { binmode => 'utf-8', normalization => 'NFD' },
+      $file });
+
+
+=item NORMALIZATION
+
+UTF-8 strings and you can customise the normalization with the NORMALIZATION option.
+
+=back
+
+
 =cut
 
 sub new
@@ -176,6 +198,7 @@ sub new
    my ($class, @source) = @_;
 
    $class = ref ($class) || $class;
+   
    my $self = {'file'     => undef,
                'type'     => undef,
                'key'      => undef,
@@ -183,8 +206,15 @@ sub new
                'metatype' => undef,
                'fields'   => [],
                'values'   => {}};
-
    bless $self, $class;
+
+   my $opts = {};
+   $opts = shift @source if scalar(@source) and ref $source[0] eq "HASH";
+   $opts->{ lc $_ } = $opts->{$_} for ( keys %$opts );
+   $self->{binmode} = 'utf-8'
+          if exists $opts->{binmode} && $opts->{binmode} =~ /utf-?8/i;
+   $self->{normalization} = $opts->{normalization} if exists $opts->{normalization};
+
    if (@source)
    {
       my $status;
@@ -199,7 +229,7 @@ sub new
             bless $self, $structure->entry_class;
          }
       }
-      elsif (@source == 2 && defined $source[0] && ! ref $source[0] && fileno ($source[1]))
+      elsif (@source == 2 && defined $source[0] && ! ref $source[0] && fileno ($source[1]) >= 0)
           { $status = $self->parse ($source[0], $source[1]) }
       elsif (@source == 1 && defined $source[0] && ! ref $source[0])
           { $status = $self->parse_s ($source[0]) }
@@ -234,6 +264,8 @@ sub clone
     $clone->{file} = $self->{file}
   }
   # These might be changed so make copies
+  $clone->{binmode} = $self->{binmode};
+  $clone->{normalization} = $self->{normalization};
   $clone->{type}     = $self->{type};
   $clone->{key}      = $self->{key};
   $clone->{status}   = $self->{status};
@@ -271,6 +303,10 @@ sub read
    my $fn = $source->{'filename'};
    my $fh = $source->{'handle'};
    $self->{'file'} = $source;        # store File object for later use
+   ## Propagate flags
+   for my $f (qw.binmode normalization.) {
+      $self->{$f} = $source->{$f} unless exists $self->{$f};
+   }
    return $self->parse ($fn, $fh, $preserve);
 }
 
@@ -392,27 +428,39 @@ context.)
 
 =item fieldlist ()
 
-Returns the list of fields in the entry.  In a scalar context, returns a
-reference to the object's own list of fields.  That way, you can change or
-reorder the field list with minimal interference from the class.  I'm not
-entirely sure if this is a good idea, so don't rely on it existing in the
-future; feel free to play around with it and let me know if you get bitten
-in dangerous ways or find this enormously useful.
+Returns the list of fields in the entry.  
+
+B<WARNING> In scalar context, it no longer returns a
+reference to the object's own list of fields.
 
 =cut
 
 sub parse_ok   { shift->{'status'}; }
 
-sub metatype   { shift->{'metatype'}; }
+sub metatype   {
+    my $self = shift;
+    Text::BibTeX->_process_result( $self->{'metatype'}, $self->{binmode}, $self->{normalization} );
+}
 
-sub type       { shift->{'type'}; }
+sub type {
+    my $self = shift;
+    Text::BibTeX->_process_result( $self->{'type'}, $self->{binmode}, $self->{normalization} );
+}
 
-sub key        { shift->{'key'}; }
+sub key        { 
+  my $self = shift;
+  exists $self->{key}
+    ? Text::BibTeX->_process_result($self->{key}, $self->{binmode}, $self->{normalization})
+    : undef;
+}
 
 sub num_fields { scalar @{shift->{'fields'}}; }
 
-sub fieldlist  { wantarray ? @{shift->{'fields'}} : shift->{'fields'}; }
-
+sub fieldlist  { 
+  my $self = shift;
+  return map { Text::BibTeX->_process_result($_, $self->{binmode}, $self->{normalization})} @{$self->{'fields'}};
+}
+  
 =item exists (FIELD)
 
 Returns true if a field named FIELD is present in the entry, false
@@ -484,17 +532,24 @@ sub exists
 {
    my ($self, $field) = @_;
 
-   exists $self->{'values'}{$field};
+   exists $self->{values}{Text::BibTeX->_process_argument($field, $self->{binmode}, $self->{normalization})};
 }
 
 sub get
 {
    my ($self, @fields) = @_;
 
-   @{$self->{'values'}}{@fields};
+   my @x = @{$self->{'values'}}{map {Text::BibTeX->_process_argument($_, $self->{binmode}, $self->{normalization})} @fields};
+
+   @x = map {defined($_) ? Text::BibTeX->_process_result($_, $self->{binmode}, $self->{normalization}): undef} @x;
+
+   return (@x > 1) ? @x : $x[0];
 }
 
-sub value { shift->{'value'} }
+sub value { 
+  my $self = shift;
+  Text::BibTeX->_process_result($self->{value}, $self->{binmode}, $self->{normalization});
+}
 
 
 =head2 Author name methods
@@ -590,17 +645,19 @@ sub split
 {
    my ($self, $field, $delim, $desc) = @_;
 
-   return unless $self->exists ($field);
+   return unless $self->exists($field);
    $delim ||= 'and';
    $desc ||= 'name';
 
-   my $filename = ($self->{'file'} && $self->{'file'}{'filename'});
-   my $line = $self->{'lines'}{$field};
-
 #   local $^W = 0                        # suppress spurious warning from 
 #      unless defined $filename;         # undefined $filename
-   Text::BibTeX::split_list ($self->{'values'}{$field}, $delim,
-                             $filename, $line, $desc);
+   Text::BibTeX::split_list($self->{values}{$field},
+                            $delim,
+                            ($self->{file} && $self->{file}{filename}),
+                            $self->{lines}{$field},
+                            $desc,
+                            {binmode       => $self->{binmode},
+                             normalization => $self->{normalization}});
 }
 
 sub names
@@ -618,7 +675,8 @@ sub names
 #      unless defined $filename;         # undefined $filename
    for $i (0 .. $#names)
    {
-      $names[$i] = Text::BibTeX::Name->new($names[$i], $filename, $line, $i);
+      $names[$i] = Text::BibTeX::Name->new(
+        {binmode => $self->{binmode}, normalization => $self->{normalization}},$names[$i], $filename, $line, $i);
    }
    @names;
 }
@@ -696,7 +754,7 @@ sub set_key
 {
    my ($self, $key) = @_;
 
-   $self->{'key'} = $key;
+   $self->{'key'} = Text::BibTeX->_process_argument($key, $self->{binmode}, $self->{normalization});
 }
 
 sub set
@@ -708,7 +766,7 @@ sub set
 
    while (@_)
    {
-      ($field,$value) = (shift,shift);
+      ($field,$value) = (shift,Text::BibTeX->_process_argument(shift, $self->{binmode}, $self->{normalization}));
       push (@{$self->{'fields'}}, $field)
          unless exists $self->{'values'}{$field};
       $self->{'values'}{$field} = $value;
@@ -870,7 +928,8 @@ sub print_s
 
    # Tack on the last line, and we're done!
    $output .= "}\n\n";
-   $output;
+   
+   Text::BibTeX->_process_result($output, $self->{binmode}, $self->{normalization});
 }
 
 =back
